from datetime import date
import auto


def get_auto_vals(ini_file, selectedStartDate, selectedEndDate):
    car = auto.Masina(ini_file)
    alimentari = car.get_alimentari_for_interval_type(selectedStartDate, selectedEndDate, None)
    for i in alimentari:
        print(i)
    # print(auto.table_totals)


def main():
    selectedStartDate = date(2021, 12, 30)
    selectedEndDate = date(2024, 1, 29)

    masina_ini = r"D:\Python\MySQL\masina.ini"
    # get_auto_vals(income_ini, selectedStartDate, selectedEndDate)
    car = auto.Masina(masina_ini)
    print(car.last_records)


if __name__ == '__main__':
    main()



import sys
import os
import traceback
import decimal
import datetime as dt
from datetime import datetime, timedelta
import numpy as np
from PyQt5.QtWidgets import *
from PyQt5 import uic
from PyQt5 import QtCore
from PyQt5.QtCore import *
import sip
import matplotlib.pyplot as plt
import csv
from masina.auto import Masina
np.set_printoptions(linewidth=600)


class MyApp(QMainWindow):
    def __init__(self, ini_file):
        super(MyApp, self).__init__()
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'masina.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.app = Masina(ini_file)

        self.populateSummaryTab()

        self.populate_CB_alim_types()
        self.populateCBMonths()
        self.alim_date()
        self.populateDatesInterval()
        self.populateDetailsTab()

        # self.get_table_info()

        # self.ui.cbActiveConto.currentIndexChanged.connect(self.get_table_info)
        # self.ui.DEFrom.dateTimeChanged.connect(self.get_table_info)
        # self.ui.DEBis.dateTimeChanged.connect(self.get_table_info)
        self.ui.PB_fire_add_row.clicked.connect(self.add_row)
        self.ui.PB_fire_filter.clicked.connect(self.populateDetailsTab)
        self.ui.TB_add_file.clicked.connect(self.get_file_pth)
        self.ui.PB_Export_csv.clicked.connect(self.export_CSV)
        self.ui.TW_all_alimentari.horizontalHeader().sectionClicked.connect(self.sortPlan)
        self.ui.TW_all_alimentari.itemDoubleClicked.connect(self.upload_download_file)

    def populate_table_widget(self, widget, table):
        tableHead, table_data = list(table[0]), table[1:]
        widget.setColumnCount(len(tableHead))
        widget.setHorizontalHeaderLabels(tableHead)
        widget.setRowCount(table_data.shape[0])
        for col in range(table_data.shape[1]):
            for row in range(table_data.shape[0]):
                # if isinstance(table_data[row, col], int) or isinstance(table_data[row, col], float):
                #     item = QTableWidgetItem()
                #     item.setData(QtCore.Qt.DisplayRole, table[row, col])
                # elif isinstance(table_data[row, col], decimal.Decimal):
                #     val = float(table_data[row, col])
                #     item = QTableWidgetItem()
                #     item.setData(QtCore.Qt.DisplayRole, val)
                # # elif tableHead[col] == 'file_name':
                # #     text = "<a href={}>{}</a>".format(str(table_data[row, col]), str(table_data[row, col]))
                # #     item = QTableWidgetItem(text)
                # #     # item.setText(text)
                # #     # item.setOpenExternalLinks(True)
                # #     # item.clicked.connect(self.download_file)
                # #     # widget.setCellWidget(row, col, item)
                # #     # continue
                # else:
                #     item = QTableWidgetItem(str(table_data[row, col]))
                item = QTableWidgetItem(str(table_data[row, col]))
                widget.setItem(row, col, item)

    def upload_download_file(self, item):
        print(item.column(), item.row(), item.text())
        if self.ui.TW_all_alimentari.horizontalHeaderItem(item.column()).text() == 'file_name':
            # print(item)
            id_ = self.ui.TW_all_alimentari.item(item.row(), 0).text()
            if item.text() != 'None':
                # print('ÄÄ', id_)
                print('id', id_)
                print('ÖÖ', self.ui.TW_all_alimentari.horizontalHeaderItem(0).text())
                expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "")
                print(expName)
                if expName:
                    file_content = self.app.alimentari.returnCellsWhere('file', ('id', id_))[0]
                    self.app.alimentari.write_file(file_content, expName)
                else:
                    return
            else:
                print('upload')
                uploadFile, _ = QFileDialog.getOpenFileName(self, "upload file", "", "")
                self.app.upload_file(uploadFile, id_)
                self.populateDetailsTab()

    def populateSummaryTab(self):
        self.populate_table_widget(self.ui.table_totals, self.app.table_totals)
        self.populate_table_widget(self.ui.table_alimentari, self.app.table_alimentari)
        self.populate_table_widget(self.ui.table_last_records, self.app.last_records)

    def populateDetailsTab(self):
        # self.populate_table_widget(self.ui.TW_all_alimentari, self.app.get_all_alimentari())
        selectedStartDate = self.ui.DEFrom.date().toPyDate()
        selectedEndDate = self.ui.DEBis.date().toPyDate()
        alim_type = self.ui.CB_alim_types_filter.currentText()
        if self.ui.CB_alim_types_filter.currentText() == 'all':
            alim_type = None
        res_table = self.app.get_alimentari_for_interval_type(selectedStartDate, selectedEndDate, alim_type)
        self.populate_table_widget(self.ui.TW_all_alimentari, res_table)
        self.ui.LE_filter_showing.setText(str(res_table.shape[0]))
        self.ui.LE_total_rows.setText(str(self.app.alimentari.noOfRows))
        return res_table

    def add_row(self):
        # print('self.app.insert_new_alim()')
        data = self.ui.alim_date.date().toPyDate()
        alim_type = self.ui.CB_alim_types.currentText()
        brutto = self.ui.LE_brutto.text()
        amount = self.ui.LE_amount.text()
        refuel = self.ui.LE_refuel.text()
        other = self.ui.LE_other.text()
        recharges = self.ui.LE_recharges.text()
        km = self.ui.LE_km.text()
        comment = self.ui.LE_comment.text()
        self.app.insert_new_alim(data, alim_type, brutto, amount, refuel, other, recharges, km, comment, self.loadFile)

    def populate_CB_alim_types(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.CB_alim_types.addItems(self.app.types_of_costs)
        self.ui.CB_alim_types_filter.addItem('all')
        self.ui.CB_alim_types_filter.addItems(self.app.types_of_costs)

    def alim_date(self):
        self.ui.alim_date.setDate(datetime.now())
        self.ui.alim_date.setCalendarPopup(True)

    def get_file_pth(self):
        self.loadFile, _ = QFileDialog.getOpenFileName(self, "File", "", "File (*.jpg;*.JPG;*.pdf)")
        if self.loadFile:
            self.ui.LE_add_file.setText(self.loadFile)

    def populateCBMonths(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.CBMonths.addItem('interval')
        months = [dt.date(2000, m, 1).strftime('%B') for m in range(1, 13)]
        for month in months:
            self.ui.CBMonths.addItem(month)

        self.ui.SB_year.setValue(datetime.now().year)

    def populateDatesInterval(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        startDate, lastDayOfMonth = self.app.default_interval

        if self.ui.CBMonths.currentText() != 'interval':
            year = int(self.ui.SB_year.value())
            startDate, lastDayOfMonth = self.app.get_monthly_interval(self.ui.CBMonths.currentText(), year)

        self.ui.DEFrom.setDate(startDate)
        self.ui.DEBis.setDate(lastDayOfMonth)

        self.ui.DEFrom.setCalendarPopup(True)
        self.ui.DEBis.setCalendarPopup(True)

    def sortPlan(self, logical_index):
        print(sys._getframe().f_code.co_name)
        header = self.ui.TW_all_alimentari.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.TW_all_alimentari.sortItems(logical_index, order)

    def export_CSV(self):
        expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "File (*.csv)")
        print(expName)
        if expName:
            res_table = self.populateDetailsTab()
            with open(expName, 'w', newline='') as file:
                writer = csv.writer(file, delimiter=';')
                # if isinstance(array, list) or isinstance(array, np.ndarray):
                for row in res_table:
                    writer.writerow(row)

    ################################################

    def delete_items_of_layout(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        layout_grupa = self.gb_available_databases.layout()
        if layout_grupa is not None:
            while layout_grupa.count():
                item = layout_grupa.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.setParent(None)
                else:
                    delete_items_of_layout(item.layout())
            sip.delete(layout_grupa)

    # def connect_to_db(self):
    #     #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     print('self.sender().objectName()', self.sender().text())
    #     self.ui.GB_planned_expenses.setVisible(True)
    #     self.populateCBMonths()
    #     self.populateDatesInterval()
    #     self.get_table_info(self.sender().text())
    #     # self.populateCBConto()

    def get_table_info(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        displayExpensesTableHead = ['table', 'name', 'value', 'myconto', 'payDay', 'freq']
        displayIncomeTableHead = ['name', 'brutto', 'lohnsteuer', 'rentenvers', 'arbeitslosvers', 'gesetzliche_abzuge',
                                  'netto', 'krankenvers', 'privatvers', 'abzuge', 'uberweisung', 'myconto', 'payDay',
                                  'freq', 'in_salary']
        selectedStartDate = self.ui.DEFrom.date().toPyDate()
        selectedEndDate = self.ui.DEBis.date().toPyDate()
        self.app.prepareTablePlan(self.ui.cbActiveConto.currentText(), selectedStartDate, selectedEndDate)
        self.income.prepareTablePlan(self.ui.cbActiveConto.currentText(), selectedStartDate, selectedEndDate)

        self.populateExpensesPlan(displayExpensesTableHead)
        self.populate_expenses_summary()
        self.populateTree()
        self.populateIncomePlan(displayIncomeTableHead)
        self.populate_income_summary()
        self.totals()

    # def export(self):
    #     #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "Excel Files (*.xlsx)")
    #     worksheets = [('Complete', datetime(datetime.now().year, 1, 1),datetime(datetime.now().year, 12, 31))]
    #     for mnth in range(1, 13):
    #         firstDayOfMonth = datetime(datetime.now().year, mnth, 1)
    #         if mnth != 12:
    #             lastDayOfMonth = datetime(datetime.now().year, mnth+1, 1) - timedelta(days=1)
    #         else:
    #             lastDayOfMonth = datetime(datetime.now().year + 1, 1, 1) - timedelta(days=1)
    #
    #         tup = (firstDayOfMonth.strftime("%B"), firstDayOfMonth, lastDayOfMonth)
    #         worksheets.append(tup)
    #
    #     wb = Workbook()
    #     ws = wb.active
    #     for mnth, firstDayOfMonth, lastDayOfMonth in worksheets:
    #         # print(mnth, firstDayOfMonth, lastDayOfMonth)
    #         if mnth == 'Complete':
    #             ws.title = mnth
    #         else:
    #             wb.create_sheet(mnth)
    #         ws = wb[mnth]
    #         self.ui.DEFrom.setDate(QDate(firstDayOfMonth))
    #         self.ui.DEBis.setDate(QDate(lastDayOfMonth))
    #         self.prepareTablePlan()
    #
    #         planExpenseTable, planExpenseTableHead = self.readPlanExpenses()
    #         cheltData = np.insert(planExpenseTable, 0, planExpenseTableHead, 0)
    #
    #         for i, row in enumerate(cheltData):
    #             for j, col in enumerate(row):
    #                 ws.cell(row=i + 1, column=j + 1).value = cheltData[i][j]
    #
    #         firstRow = 1
    #         firstCol = get_column_letter(1)
    #         lastRow = len(cheltData)
    #         lastCol = get_column_letter(len(cheltData[0]))
    #
    #         table_title = '{}_{}'.format('chelt', mnth )
    #         new_text = ('{}{}:{}{}'.format(firstCol, firstRow, lastCol, lastRow))
    #         tab = Table(displayName=table_title, ref=new_text)
    #         # Add a default style with striped rows and banded columns
    #         style = TableStyleInfo(name="TableStyleMedium9", showFirstColumn=False,
    #                                showLastColumn=False, showRowStripes=True, showColumnStripes=True)
    #         tab.tableStyleInfo = style
    #         ws.add_table(tab)
    #         ws.cell(row=lastRow + 1, column=1).value = 'Total Number of Expenses'
    #         ws.cell(row=lastRow + 1, column=2).value = self.ui.LEtotalNoOfTransactions.text()
    #         ws.cell(row=lastRow + 2, column=1).value = 'Total Expenses'
    #         ws.cell(row=lastRow + 2, column=2).value = self.ui.LEtotalValue.text()
    #         #######income
    #
    #         planIncomeTable, planIncomeTableHead = self.readPlanIncome()
    #         incomeData = np.insert(planIncomeTable, 0, planIncomeTableHead, 0)
    #         firstRow = lastRow + 5
    #         firstCol = get_column_letter(1)
    #         lastRow = firstRow + len(incomeData)
    #         lastCol = get_column_letter(len(incomeData[0]))
    #
    #         for i, row in enumerate(incomeData):
    #             for j, col in enumerate(row):
    #                 ws.cell(row=i + firstRow, column=j + 1).value = incomeData[i][j]
    #
    #         table_title = '{}_{}'.format('income', mnth )
    #         new_text1 = ('{}{}:{}{}'.format(firstCol, firstRow, lastCol, lastRow))
    #         tab = Table(displayName=table_title, ref=new_text1)
    #         # Add a default style with striped rows and banded columns
    #         style = TableStyleInfo(name="TableStyleMedium9", showFirstColumn=False,
    #                                showLastColumn=False, showRowStripes=True, showColumnStripes=True)
    #         tab.tableStyleInfo = style
    #         ws.add_table(tab)
    #         ws.cell(row=lastRow + 1, column=1).value = 'Total Number of Incomes'
    #         ws.cell(row=lastRow + 1, column=2).value = self.ui.LEtotalNoOfIncome.text()
    #         ws.cell(row=lastRow + 2, column=1).value = 'Total Income'
    #         ws.cell(row=lastRow + 2, column=2).value = self.ui.LEtotalIncome.text()
    #
    #     wb.save(expName)

    def populateTree(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.TWmnthVSIrreg.clear()
        self.ui.TWmnthVSIrreg.setHeaderLabels(['freq', 'name', 'value'])
        monthly_level = QTreeWidgetItem(self.ui.TWmnthVSIrreg)
        monthly_level.setText(0, 'Monthly')
        irregular_level = QTreeWidgetItem(self.ui.TWmnthVSIrreg)
        irregular_level.setText(0, 'Irregular')
        monthlyIndx = np.where(self.app.expenses[:, self.app.tableHead.index('freq')] == 1)
        monthly = self.app.expenses[monthlyIndx]
        for mnth in monthly:
            mnth_item_level = QTreeWidgetItem(monthly_level)
            mnth_item_level.setText(1, mnth[self.app.tableHead.index('name')])
            mnth_item_level.setText(2, str(round(mnth[self.app.tableHead.index('value')])))

        totalMonthly = self.app.expenses[monthlyIndx,self.app.tableHead.index('value')][0]
        monthly_level.setText(1, 'Total')
        monthly_level.setText(2, str(round(sum(totalMonthly), 2)))

        irregIndx = np.where(self.app.expenses[:, self.app.tableHead.index('freq')] != 1)
        irregular = self.app.expenses[irregIndx]
        for irr in irregular:
            irr_item_level = QTreeWidgetItem(irregular_level)
            irr_item_level.setText(1, irr[self.app.tableHead.index('name')])
            irr_item_level.setText(2, str(round(irr[self.app.tableHead.index('value')], 2)))

        totalIrreg = self.app.expenses[irregIndx,self.app.tableHead.index('value')][0]
        irregular_level.setText(1, 'Total')
        irregular_level.setText(2, str(round(sum(totalIrreg), 2)))

    def populateExpensesPlan(self, displayTableHead=None):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if displayTableHead:
            tableHead, table = self.app.convert_to_display_table(self.app.tableHead, self.app.expenses, displayTableHead)
        else:
            tableHead, table = self.app.tableHead, self.app.expenses

        self.ui.planTable.setColumnCount(len(tableHead))
        self.ui.planTable.setHorizontalHeaderLabels(tableHead)
        self.ui.planTable.setRowCount(table.shape[0])
        for col in range(table.shape[1]):
            for row in range(table.shape[0]):
                if isinstance(table[row, col], int) or isinstance(table[row, col], float):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table[row, col])
                elif isinstance(table[row, col], decimal.Decimal):
                    val = float(table[row, col])
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, val)
                else:
                    item = QTableWidgetItem(str(table[row, col]))
                self.ui.planTable.setItem(row, col, item)

    def populate_expenses_summary(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.LEtotalNoOfTransactions.setText(str(self.app.tot_no_of_expenses()))
        self.ui.LEtotalValue.setText(str(self.app.tot_val_of_expenses()))
        self.ui.LEnoOfMonthly.setText(str(self.app.tot_no_of_monthly_expenses()))
        self.ui.LEtotalMonthly.setText(str(self.app.tot_val_of_monthly_expenses()))
        self.ui.LEnoOfIrregular.setText(str(self.app.tot_no_of_irregular_expenses()))
        self.ui.LEirregular.setText(str(self.app.tot_val_of_irregular_expenses()))

    def populateIncomePlan(self, displayTableHead=None):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if displayTableHead:
            tableHead, table = self.income.convert_to_display_table(self.income.tableHead, self.income.income, displayTableHead)
        else:
            tableHead, table = self.income.tableHead, self.income.income

        self.ui.planTableIncome.setColumnCount(len(tableHead))
        self.ui.planTableIncome.setHorizontalHeaderLabels(tableHead)
        self.ui.planTableIncome.setRowCount(table.shape[0])
        for col in range(table.shape[1]):
            for row in range(table.shape[0]):
                if isinstance(table[row, col], int) or isinstance(table[row, col], float):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table[row, col])
                elif isinstance(table[row, col], decimal.Decimal):
                    val = float(table[row, col])
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, val)
                else:
                    item = QTableWidgetItem(str(table[row, col]))
                self.ui.planTableIncome.setItem(row, col, item)

    def populate_income_summary(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.LE_total_netto.setText(str(self.income.netto))
        self.ui.LE_total_taxes.setText(str(self.income.taxes))
        self.ui.LE_total_brutto.setText(str(self.income.brutto))
        self.ui.LE_salary_uberweisung.setText(str(self.income.salary_uberweisung))
        self.ui.LE_salary_abzuge.setText(str(self.income.salary_abzuge))
        self.ui.LE_salary_netto.setText(str(self.income.salary_netto))
        self.ui.LE_salary_gesetzliche_abzuge.setText(str(self.income.salary_gesetzliche_abzuge))
        self.ui.LE_salary_brutto.setText(str(self.income.salary_brutto))

    def totals(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        total = round(self.app.tot_val_of_expenses() + self.income.netto, 2)
        self.ui.LEtotalVa.setText(str(total))

    # def sortPlan(self, logical_index):
    #     #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     header = self.ui.TW_all_alimentari.horizontalHeader()
    #     order = Qt.DescendingOrder
    #     if not header.isSortIndicatorShown():
    #         header.setSortIndicatorShown(True)
    #     elif header.sortIndicatorSection() == logical_index:
    #         order = header.sortIndicatorOrder()
    #     header.setSortIndicator(logical_index, order)
    #     self.ui.TW_all_alimentari.sortItems(logical_index, order)

    def readPlanExpenses(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.ui.planTable.rowCount()
        cols = self.ui.planTable.columnCount()
        planExpenseTable = np.empty((rows, cols), dtype=object)
        planExpenseTableHead = []
        for row in range(rows):
            for column in range(cols):
                cell = self.ui.planTable.item(row, column)
                planExpenseTable[row, column] = cell.text()
                colName = self.ui.planTable.horizontalHeaderItem(column).text()
                if colName not in planExpenseTableHead:
                    planExpenseTableHead.append(colName)

        return planExpenseTable, planExpenseTableHead

    def readPlanIncome(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = self.ui.planTableIncome.rowCount()
        cols = self.ui.planTableIncome.columnCount()
        planIncomeTable = np.empty((rows, cols), dtype=object)
        planIncomeTableHead = []
        for row in range(rows):
            for column in range(cols):
                cell = self.ui.planTableIncome.item(row, column)
                planIncomeTable[row, column] = cell.text()
                colName = self.ui.planTableIncome.horizontalHeaderItem(column).text()
                if colName not in planIncomeTableHead:
                    planIncomeTableHead.append(colName)

        return planIncomeTable, planIncomeTableHead

    def plotTablePie(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
        allValues = realExpenseTable[:, realExpenseTableHead.index('value')].astype(float)
        if None in allValues:
            allValues = allValues[allValues != np.array(None)]
        totalVal = sum(allValues)

        colTableName = realExpenseTable[:, realExpenseTableHead.index('table')]
        labels = []
        sizes = []
        for table in np.unique(colTableName):
            indx = np.where(realExpenseTable[:, realExpenseTableHead.index('table')]==table)
            smallArray = realExpenseTable[indx]
            values = sum(smallArray[:, realExpenseTableHead.index('value')].astype(float))
            txt = '{} = {:.2f}'.format(table, values)
            labels.append(txt)
            size = (values/totalVal)*100
            sizes.append(size)

        fig1, ax1 = plt.subplots()
        ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
        ax1.axis('equal')
        plt.legend(title='Total: {:.2f}'.format(totalVal))

        plt.show()

    def plotNamePie(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
        allValues = realExpenseTable[:, realExpenseTableHead.index('value')].astype(float)
        if None in allValues:
            allValues = allValues[allValues != np.array(None)]
        totalVal = sum(allValues)

        colTableName = realExpenseTable[:, realExpenseTableHead.index('name')]
        labels = []
        sizes = []
        for table in np.unique(colTableName):
            indx = np.where(realExpenseTable[:, realExpenseTableHead.index('name')]==table)
            smallArray = realExpenseTable[indx]
            values = sum(smallArray[:, realExpenseTableHead.index('value')].astype(float))
            txt = '{} = {:.2f}'.format(table, values)
            labels.append(txt)
            size = (values/totalVal)*100
            sizes.append(size)

        fig1, ax1 = plt.subplots()
        ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
        ax1.axis('equal')
        plt.legend(title='Total: {:.2f}'.format(totalVal))

        plt.show()

    def plotGraf(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
        planIncomeTable, planIncomeTableHead = self.readPlanIncome()
        x_exp = []
        y_exp = []
        for date in np.unique(realExpenseTable[:, realExpenseTableHead.index('payDay')]):
            indx = np.where(realExpenseTable[:, realExpenseTableHead.index('payDay')] == date)
            arr = realExpenseTable[indx, realExpenseTableHead.index('value')].astype(float)
            x_exp.append(date)
            y_exp.append(abs(sum(arr[0])))

        x_inc = []
        y_inc = []
        for date in np.unique(planIncomeTable[:, planIncomeTableHead.index('payDay')]):
            indx = np.where(planIncomeTable[:, planIncomeTableHead.index('payDay')] == date)
            arr = planIncomeTable[indx, planIncomeTableHead.index('value')].astype(float)
            x_inc.append(date)
            y_inc.append(abs(sum(arr[0])))

        fig1, ax1 = plt.subplots()
        ax1.plot(x_exp, y_exp)
        ax1.plot(x_inc, y_inc)
        # plt.setp(plt.get_xticklabels(), rotation=30, ha="right")
        fig1.autofmt_xdate()
        plt.grid()
        plt.show()


def main():
    ini_file = r"D:\Python\MySQL\masina.ini"
    app = QApplication(sys.argv)
    window = MyApp(ini_file)
    window.show()
    # sys.exit(app.exec_())
    app.exec_()
    #todo last 5 charges/benzin/etc in summary


if __name__ == '__main__':
    main()



import numpy as np
from mysqlquerys import connect
from mysqlquerys import mysql_rm
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import traceback
import sys, os


class Masina:
    def __init__(self, ini_file, table_name='hyundai_ioniq'):
        '''
        :param ini_file:type=QFileDialog.getOpenFileName name=filename file_type=(*.ini;*.txt)
        '''
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if isinstance(ini_file, dict):
            credentials = ini_file
        else:
            self.conf = connect.Config(ini_file)
            credentials = self.conf.credentials
        self.alimentari = mysql_rm.Table(credentials, table_name)
        self.types_of_costs = ["electric", "benzina", "intretinere", "asigurare", 'impozit', 'TüV']
        # try:
        #     self.dataBase = self.sql_rm.DataBase(self.conf.credentials)
        # except Exception as err:
        #     print(traceback.format_exc())

    # @property
    # def sql_rm(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     if self.conf.db_type == 'mysql':
    #         sql_rm = mysql_rm
    #     return sql_rm

    @property
    def no_of_records(self):
        return self.alimentari.noOfRows

    @property
    def default_interval(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        startDate = datetime(datetime.now().year - 1, datetime.now().month, datetime.now().day)
        endDate = datetime(datetime.now().year, datetime.now().month, datetime.now().day)
        return startDate, endDate

    @property
    def total_money(self):
        col = self.alimentari.returnColumn('brutto')
        return round(sum(col), 2)

    @property
    def tot_benzina(self):
        matches = [('type', 'benzina')]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    @property
    def tot_electric(self):
        matches = [('type', 'electric')]
        col = self.alimentari.returnCellsWhere('brutto', matches)
        return round(sum(col), 2)

    @property
    def monthly(self):
        return round((self.monthly_benzina+self.monthly_electric), 2)

    @property
    def monthly_benzina(self):
        matches = [('type', 'benzina')]
        money = self.alimentari.returnCellsWhere('brutto', matches)
        all_dates = self.alimentari.returnColumn('data')
        start_date = min(all_dates)
        finish_date = max(all_dates)
        total_money = round(sum(money), 2)
        days = (finish_date - start_date).days
        average_day_per_month = 365/12
        monthly = (average_day_per_month * total_money) / days
        return round(monthly, 2)

    @property
    def monthly_electric(self):
        matches = [('type', 'electric')]
        money = self.alimentari.returnCellsWhere('brutto', matches)
        all_dates = self.alimentari.returnColumn('data')
        start_date = min(all_dates)
        finish_date = max(all_dates)
        total_money = round(sum(money), 2)
        days = (finish_date - start_date).days
        average_day_per_month = 365/12
        monthly = (average_day_per_month * total_money) / days
        return round(monthly, 2)

    @property
    def db_start_date(self):
        all_dates = self.alimentari.returnColumn('data')
        start_date = min(all_dates)
        return start_date

    @property
    def db_last_record_date(self):
        all_dates = self.alimentari.returnColumn('data')
        finish_date = max(all_dates)
        return finish_date

    @property
    def table_alimentari(self):
        arr = [('', 'Alimentari[€]', 'Benzina[€]', 'Electric[€]')]
        if self.no_of_records > 0:
            total_alim = self.tot_benzina + self.tot_electric
            arr.append(('Monthly', self.monthly, self.monthly_benzina, self.monthly_electric))
            arr.append(('Total', total_alim, self.tot_benzina, self.tot_electric))
        else:
            arr.append(('Monthly', None, None, None))
            arr.append(('Total', None, None, None))

        arr = np.atleast_2d(arr)
        return arr

    @property
    def table_totals(self):
        types = ['benzina', 'electric', 'asigurare', 'impozit', 'TüV', 'intretinere']
        table = []
        for year in reversed(range(self.db_start_date.year, self.db_last_record_date.year+1)):
            # print(year)
            dd = {}
            dd['year'] = year
            startTime = datetime(year, 1, 1)
            endTime = datetime(year+1, 1, 1)
            rows = self.alimentari.returnRowsOfYear('data', startTime, 'data', endTime)
            arr = np.atleast_2d(rows)
            tot = 0
            for t in types:
                indx = np.where(arr[:,self.alimentari.columnsNames.index('type')] == t)
                col = arr[indx, self.alimentari.columnsNames.index('brutto')]
                value = sum(col[0])
                value = round(value, 2)
                dd[t] = value
                tot += value
            dd['total/row'] = round(tot, 2)
            table.append(dd)
        table_head = tuple(dd.keys())
        arr = [table_head]
        for tab in table:
            row = []
            for k, v in tab.items():
                row.append(v)
            arr.append(tuple(row))
        arr = np.atleast_2d(arr)
        row_totals = ['totals']
        total_total = 0
        for col in range(1, arr.shape[1]):
            # print(arr[0, col], round(sum(arr[1:, col].astype(float)), 2))
            val = round(sum(arr[1:, col].astype(float)), 2)
            row_totals.append(val)
            total_total += val
        row_tot = np.array(row_totals)
        new_arr = np.insert(arr, 1, row_tot, axis=0)
        return new_arr

    @property
    def last_records(self):
        table_head = self.alimentari.columnsNames
        table_head.remove('file')
        last_records = [table_head]
        for typ in self.types_of_costs:
            matches = [('type', typ)]
            table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))
            if typ == 'electric':
                already_listed = []
                for row in table:
                    provider = row[table_head.index('comment')]
                    if provider not in already_listed:
                        last_records.append(row)
                        already_listed.append(provider)
                continue
            if table:
                last_records.append(table[0])

        last_records = np.atleast_2d(last_records)
        # for i in last_records:
        #     print(i)
        # print(50*'+')
        return last_records

    def get_monthly_interval(self, month:str, year):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        mnth = datetime.strptime(month, "%B").month
        startDate = datetime(year, mnth, 1)

        if mnth != 12:
            lastDayOfMonth = datetime(year, mnth + 1, 1) - timedelta(days=1)
        else:
            lastDayOfMonth = datetime(year + 1, 1, 1) - timedelta(days=1)

        return startDate, lastDayOfMonth

    def get_all_alimentari(self):
        cols = []
        for k, v in self.alimentari.columnsDetProperties.items():
            if v[0] == 'longblob':
                continue
            cols.append(k)
        alimentari = self.alimentari.returnColumns(cols)
        # alimentari = self.alimentari.returnAllRecordsFromTable()
        alimentari = np.atleast_2d(alimentari)
        alimentari = np.insert(alimentari, 0, cols, axis=0)
        return alimentari

    def get_alimentari_for_interval_type(self, selectedStartDate, selectedEndDate, alim_type):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        matches = [('data', (selectedStartDate, selectedEndDate))]
        if alim_type:
            matches.append(('type', alim_type))
        # print(matches)
        table = self.alimentari.filterRows(matches, order_by=('data', 'DESC'))

        if table:
            table_head = []
            for col_name, prop in self.alimentari.columnsDetProperties.items():
                # print(col_name, prop)
                if prop[0] == 'longblob':
                    continue
                table_head.append(col_name)
            arr = np.atleast_2d(table)
            arr = np.insert(arr, 0, np.array(table_head), axis=0)
        else:
            arr = np.atleast_2d(np.array(self.alimentari.columnsNames))
        return arr

    def upload_file(self, file, id):
        print(file, id)
        self.alimentari.changeCellContent('file', file, 'id', id)
        _, file_name = os.path.split(file)
        self.alimentari.changeCellContent('file_name', file_name, 'id', id)
        # self.alimentari.addNewRow(cols, vals)

    def insert_new_alim(self, data, alim_type, brutto, amount, refuel, other, recharges, km, comment, file):
        '''
        :param data:type=dateTime name=date
        :param alim_type:type=comboBox name=alim_type items=[electric,benzina,TüV,intretinere]
        :param brutto:type=text name=brutto
        :param amount:type=text name=amount
        :param refuel:type=text name=refuel
        :param other:type=text name=other
        :param recharges:type=text name=recharges
        :param km:type=text name=km
        :param comment:type=text name=comment
        :param file:type=QFileDialog.getOpenFileName name=file
        '''
        _, file_name = os.path.split(file)
        cols = ['data', 'type', 'brutto', 'amount', 'refuel', 'other', 'recharges', 'ppu', 'km', 'comment', 'file', 'file_name']
        try:
            if isinstance(brutto, str) and ',' in brutto:
                brutto = brutto.replace(',', '.')
            brutto = float(brutto)
        except:
            brutto = None
        try:
            if isinstance(amount, str) and ',' in amount:
                amount = amount.replace(',', '.')
            amount = float(amount)
        except:
            amount = None
        try:
            if isinstance(refuel, str) and ',' in refuel:
                refuel = refuel.replace(',', '.')
            refuel = float(refuel)
        except:
            refuel = None
        try:
            if isinstance(other, str) and ',' in other:
                other = other.replace(',', '.')
            other = float(other)
        except:
            other = None
        try:
            if isinstance(recharges, str) and ',' in recharges:
                recharges = recharges.replace(',', '.')
            recharges = float(recharges)
        except:
            recharges = None
        try:
            km = int(km)
        except:
            km = None

        ppu = round(float(brutto) / float(amount), 3)
        vals = [data, alim_type, brutto, amount, refuel, other, recharges, ppu, km, comment, file, file_name]

        # for i in range(len(cols)):
        #     print(cols[i], vals[i], type(vals[i]))
        self.alimentari.addNewRow(cols, vals)

    def create_sql_table(self, table_name):
        masina_sql = r'static\sql\auto.sql'
        mysql_rm.DataBase(self.conf.credentials).createTableFromFile(masina_sql, table_name)



from datetime import datetime
import traceback
import sys
import re
import os
import csv
from flask import Flask, render_template, request, redirect, url_for, session
from flask_login import UserMixin, login_user, LoginManager, login_required, logout_user, current_user
from flask import send_file, send_from_directory, Response
from werkzeug.security import generate_password_hash, check_password_hash
from werkzeug.utils import secure_filename
from mysqlquerys import connect
from mysqlquerys import mysql_rm
from auto import Masina


UPLOAD_FOLDER = r"D:\Python\test"
ALLOWED_EXTENSIONS = {'txt', 'pdf', 'png', 'jpg', 'jpeg', 'gif'}

app = Flask(__name__)
# app.config['SQLALCHEMY_DATABASE_URI'] = "mysql://newuser_radu:Paroladetest_1234@localhost/cheltuieli_desktop"
app.config['SECRET_KEY'] = "my super secret"
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER


login_manager = LoginManager()
login_manager.init_app(app)
# login_manager.login_view = 'login'

# ini_file = '/home/radum/rmasina/static/wdb.ini'
ini_file = r"D:\Python\MySQL\masina.ini"
report_file = '/home/radum/rmasina/report.csv'

create_auto_table = '/home/radum/rmasina/static/sql/auto_template.sql'

conf = connect.Config(ini_file)


class Users(UserMixin):
    def __init__(self, user_name):
        self.user_name = user_name
        self.db = mysql_rm.DataBase(conf.credentials)
        self.users_table = mysql_rm.Table(conf.credentials, 'users')
        self.all_cars_table = mysql_rm.Table(conf.credentials, 'all_cars')
        # print(25*'#####')

    @property
    def id(self):
        matches = ('username', self.user_name)
        user_id = self.users_table.returnCellsWhere('id', matches)[0]
        return user_id

    @property
    def valid_user(self):
        all_users = self.users_table.returnColumn('username')
        if self.user_name in all_users:
            return True
        else:
            return False

    @property
    def admin(self):
        if self.valid_user:
            matches = ('id', self.id)
            user_type = self.users_table.returnCellsWhere('user_type', matches)[0]
            if user_type == 'admin':
                return True
            else:
                return False

    @property
    def masini(self):
        # print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        masini = {}
        matches = ('user_id', self.id)
        cars_rows = self.all_cars_table.returnRowsWhere(matches)
        if cars_rows:
            for row in cars_rows:
                indx_brand = self.all_cars_table.columnsNames.index('brand')
                indx_model = self.all_cars_table.columnsNames.index('model')
                table_name = '{}_{}'.format(row[indx_brand], row[indx_model])
                app_masina = Masina(conf.credentials, table_name=table_name.lower())
                masini[table_name] = app_masina
        return masini

    @property
    def hashed_password(self):
        matches = ('username', self.user_name)
        hashed_password = self.users_table.returnCellsWhere('password', matches)[0]
        return hashed_password

    def verify_password(self, password):
        return check_password_hash(self.hashed_password, password)

    def add_car(self, brand, model, car_type):
        cols = ('user_id', 'brand', 'model', 'cartype')
        vals = (self.id, brand, model, car_type)
        self.all_cars_table.addNewRow(cols, vals)
        newTableName = '{}_{}'.format(brand, model)
        # pth = os.path.join(os.path.dirname(__file__), create_auto_table)
        self.db.createTableFromFile(create_auto_table, newTableName.lower())

    def get_id_all_cars(self, table_name):
        brand, model = table_name.split('_')
        matches = [('user_id', self.id),
                   ('brand', brand),
                   ('model', model),
                   ]
        print(matches)
        id_all_cars = self.all_cars_table.returnCellsWhere('id', matches)[0]
        return id_all_cars


@login_manager.user_loader
def load_user(user_id):
    print(25*'Ä')
    print(sys._getframe().f_code.co_name, request.method)
    print('##**##user_id _{}_'.format(user_id), type(user_id))
    try:
        mat = int(user_id)
    except:
        mat = int(re.findall("\[(.+?)\]", user_id)[0])

    matches = ('id', mat)
    users_table = mysql_rm.Table(conf.credentials, 'users')
    name = users_table.returnCellsWhere('username', matches)[0]

    user = Users(name)
    # print('##user.user_access', user.admin)
    print(25 * 'Ä')
    return user


@app.route('/', methods=['GET', 'POST'])
def index():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        return render_template('index.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route("/login", methods=["GET", "POST"])
def login():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        if request.method == "POST":
            username = request.form['username']
            user = Users(username)
            if not user.valid_user:
                print('User {} not in database'.format(username))
                return redirect(url_for("login"))
            if user.verify_password(request.form['password']):
                login_user(user)
                return redirect(url_for("index"))
        return render_template("login.html")
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/logout', methods=['GET', 'POST'])
@login_required
def logout():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        logout_user()
        return redirect(url_for('index'))
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/register', methods=['GET', 'POST'])
def register():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        conf = connect.Config(ini_file)
        users_table = mysql_rm.Table(conf.credentials, 'users')
        if request.method == 'POST':
            username = request.form['username']
            password = request.form['password']
            cols = ('username', 'password')
            hash = generate_password_hash(password)
            vals = (username, hash)
            users_table.addNewRow(cols, vals)
            user = Users(username)
            login_user(user)
            return redirect(url_for("index"))
        elif request.method == 'GET':
            print('****', request.method)

        return render_template('register.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/car/<table_name>', methods=['GET', 'POST'])
def car(table_name):
    print('Module: {}, Def: {}, Caller: {}'.format(__name__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
    try:
        print(sys._getframe().f_code.co_name, request.method)
        if request.method == 'POST':
            if 'export_detail_table_as_csv' in request.form:
                app_masina = Masina(conf.credentials, table_name=table_name.lower())
                dataFrom, dataBis = app_masina.default_interval
                alim_type = None
                alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
                exact_path = "report.csv"
                with open(exact_path, 'w', newline='') as file:
                    writer = csv.writer(file, delimiter=';')
                    for row in alimentari:
                        writer.writerow('rrrr')
                        writer.writerow(row)
                return send_file(exact_path, as_attachment=True)

        app_masina = Masina(conf.credentials, table_name=table_name.lower())
        session['current_table'] = table_name.lower()
        session['types_of_costs'] = app_masina.types_of_costs
        if app_masina.no_of_records == 0:
            # return "<h1>Please insert some data</h1>"
            session['current_table'] = table_name.lower()
            session['types_of_costs'] = app_masina.types_of_costs
            return redirect(url_for("addalim"))
        dataFrom, dataBis = app_masina.default_interval
        alim_type = None
        alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
        return render_template('masina.html',
                               userDetails=alimentari,
                               dataFrom=dataFrom.date(),
                               dataBis=dataBis.date(),
                               tabel_alimentari=app_masina.table_alimentari,
                               tabel_totals=app_masina.table_totals,
                               types_of_costs=app_masina.types_of_costs,
                               table_name = table_name,
                               last_records=app_masina.last_records
                               )
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)

@app.route('/addalim', methods=['GET', 'POST'])
def addalim():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        if request.method == 'POST':
            if "add_alim" in request.form:
                date = request.form['data']
                alim_type = request.form['type']
                brutto = request.form['brutto']
                amount = request.form['amount']
                refuel = request.form['refuel']
                other = request.form['other']
                recharges = request.form['recharges']
                km = request.form['km']
                comment = request.form['Comment']
                file = request.files['rfile']
                fl = None
                if file:
                    file.save(secure_filename(file.filename))
                    fl = file.filename
                id_all_cars = current_user.get_id_all_cars(session['current_table'].lower())
                app_masina = Masina(conf.credentials, table_name=session['current_table'].lower())
                app_masina.insert_new_alim(current_user.id, id_all_cars, data=date, alim_type=alim_type, brutto=brutto, amount=amount,
                refuel=refuel, other=other, recharges=recharges, km=km, comment=comment, file=fl)
                return redirect(url_for('car', table_name=session['current_table'].lower()))
        return render_template('addalim.html',
                               table_name=session['current_table'],
                               types_of_costs=session['types_of_costs']
                               )
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/masina', methods=['GET', 'POST'])
def masina():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        credentials = conf.credentials
        # credentials['database'] = 'masina'
        # page = request.args.get('page')
        # print('####credentials', credentials)
        # print('*****conf.credentials', current_user.user_name)
        # print('*****request.form', request.form)
        # print('?????request.args', request.full_path)
        # print('?????request.args', request.values)
        # print('?????request.args', request.get_full_path)
        # print('?????request.args', request.path)
        # app_masina = Masina(ini_file)
        app_masina = Masina(credentials, table_name='hyundai_ioniq')
        dataFrom, dataBis = app_masina.default_interval
        alim_type = None
        if request.method == 'POST':
            print('LLLLLrequest.form', request.form)
            if "filter" in request.form:
                month = request.form['month']
                year = int(request.form['year'])
                alim_type = request.form['type']
                if alim_type == 'all':
                    alim_type = None
                dataFrom = request.form['dataFrom']
                dataBis = request.form['dataBis']

                if month != 'interval':
                    dataFrom, dataBis = app_masina.get_monthly_interval(month, year)
                elif month == 'interval' and (dataFrom == '' or dataBis == ''):
                    dataFrom, dataBis = app_masina.default_interval
                else:
                    try:
                        dataFrom = datetime.strptime(dataFrom, "%Y-%m-%d")
                        dataBis = datetime.strptime(dataBis, "%Y-%m-%d")
                    except:
                        print(traceback.format_exc())
            elif "add_alim" in request.form:
                date = request.form['data']
                alim_type = request.form['type']
                brutto = request.form['brutto']
                amount = request.form['amount']
                km = request.form['km']
                comment = request.form['Comment']
                file = request.files['rfile']
                file.save(secure_filename(file.filename))
                fl = file.filename
                fl = app_masina.alimentari.convertToBinaryData(fl)

                ppu = round(float(brutto)/float(amount), 3)
                columns = ['data', 'type', 'brutto', 'amount', 'ppu', 'km', 'comment', 'file']
                values = [date, alim_type, brutto, amount, ppu, km, comment, fl]
                redirect(url_for('masina'))
                app_masina.insert_new_alim(columns, values)
            elif 'read_txt_file_content' in request.form:
                file = request.files['myfile']
                filename = secure_filename(file.filename)
                print('filename', filename)
                fl = os.path.join("D:\Python\MySQL", filename)
                file.save(fl)
                with open(fl) as f:
                    file_content = f.read()
                    print(file_content)
                return file_content
            elif 'save_file_locally' in request.form:
                file = request.files['myfile']
                # file = 'test.csv'

                print('file', file, type(file))
                filename = secure_filename(file.filename)

                file.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))

                print('filename', filename, type(filename))
            elif 'upload_file_to_db' in request.form:
                file = request.files['myfile']
                # print('file', file, type(file))
                file.save(secure_filename(file.filename))
                fl = file.filename
                # file.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
                #
                # print('filename', filename, type(filename))
                # fl = os.path.normpath(filename)
                fl = app_masina.alimentari.convertToBinaryData(fl)
                app_masina.alimentari.changeCellContent('file', fl, 'id', 2)
                # print('filename', filename)
                # fl = os.path.join("D:\Python\MySQL", filename)
                # file.save(fl)
                # with open(fl) as f:
                #     file_content = f.read()
                #     print(file_content)
                # return file_content
            # elif 'download_file' in request.form:
            #     print('AAAAAAAAAA')
            #     fl = app_masina.alimentari.returnCellsWhere('file', ('id', 651))
            #     exact_path = "report.csv"
            #
            #     with open(exact_path, 'wb') as file:
            #         file.write(fl[0])
            #     print('BBBBBBBBBBB')
            #     return send_file(exact_path, as_attachment=True)
            #     # return send_from_directory(app.config['UPLOAD_FOLDER'], filename)
            elif 'export_detail_table_as_csv' in request.form:
                alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
                exact_path = "report.csv"
                with open(exact_path, 'w', newline='') as file:
                    writer = csv.writer(file, delimiter=';')
                    for row in alimentari:
                        writer.writerow(row)
                return send_file(exact_path, as_attachment=True)
            else:
                print("AAAA")
        print('*****dataFrom', dataFrom)
        print('*****dataBis', dataBis)
        print('*****alim_type', alim_type)
        alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
        return render_template('masina.html',
                               userDetails=alimentari,
                               dataFrom=dataFrom.date(),
                               dataBis=dataBis.date(),
                               tabel_alimentari=app_masina.table_alimentari,
                               tabel_totals=app_masina.table_totals,
                               types_of_costs=app_masina.types_of_costs,
                               last_records=app_masina.last_records
                               )
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/addcar', methods=['GET', 'POST'])
def addcar():
    try:
        print(sys._getframe().f_code.co_name, request.method)
        if request.method == 'POST':
            brand = request.form['brand']
            model = request.form['model']
            car_type = request.form['car_type']
            current_user.add_car(brand, model, car_type)
            return redirect(url_for('index'))
        return render_template('addcar.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/addfile/<index>', methods=['GET', 'POST'])
def addfile(index):
    try:
        credentials = conf.credentials
        app_masina = Masina(credentials, table_name=session['current_table'].lower())
        if request.method == 'POST':
            file = request.files['myfile']
            name = file.filename
            app_masina.alimentari.changeCellContent('file_name', str('file_{}'.format(name)), 'id', index)
            #return "UPLOAD {} {} {} ++{}++".format(index, request.method, file.filename, )
            file.save(secure_filename(name))
            fl = file.filename
            fl = app_masina.alimentari.convertToBinaryData(fl)

            app_masina.upload_file(fl, index, name)
            return redirect(url_for('car', table_name=session['current_table'].lower()))
            #return "UPLOAD {} {} {}".format(index, request.method, secure_filename(file.filename))
        elif request.method == 'GET':
            file_content = app_masina.alimentari.returnCellsWhere('file', ('id', index))[0]
            exact_path = report_file
            with open(exact_path, 'wb') as file:
                file.write(file_content)
            return send_file(exact_path, as_attachment=True)

    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/importtable', methods=['GET', 'POST'])
def importtable():
    try:
        if request.method == 'POST':
            credentials = conf.credentials
            app_masina = Masina(credentials, table_name=session['current_table'].lower())
            if 'import_table' in request.form:
                file = request.files['myfile']
                filename = secure_filename(file.filename)

                file.save(filename)
#                with open(filename) as f:
#                    file_content = f.readlines()
                file_content = []
                with open(filename, 'r') as file:
                    reader = csv.reader(file, delimiter=';')
                    for i, row in enumerate(reader):
                        print(i, row)
                        file_content.append(row)

                columns, rows = file_content[0], file_content[1:]
                for row in rows:
                    app_masina.alimentari.addNewRow(columns[1:-1], row[1:-1])

                #return file_content[0]
                return redirect(url_for('car', table_name=session['current_table'].lower()))
        elif request.method == 'GET':
            return render_template('importtable.html')

    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)




if __name__ == "__main__":
    app.run(debug=True)
